!====================================================================================
!       CA_STRUCTS_SIN.INC   Structure definitions for cryosat utilities - 1200 bytes
!===================================================================================
!
       type reaa
        integer(kind=4) ::    sec2000        ! atomic seconds since 2000  
        integer(kind=4) ::    microsec       ! microseconds
        integer(kind=4) ::    kframe         ! source sequence number 0-16383 then cycles to 0
        integer(kind=4) ::    alt            ! orbit altitude, COG above reference ellipsoid, mm 
        integer(kind=4) ::    alt_rate       ! orbit altitude rate, mm/s 
        integer(kind=4) ::    range          ! range, COG at Ku band, mm; (Cryosat from uncorrected two-way window delay) 
        integer(kind=4) ::    range_s        ! range, COG at S band, mm 
        integer(kind=4) ::    trackbits      ! Useful flags to be defined as we go along; initially zero
                                             ! 2                               1 - record empty based on no time record
                                             ! 4                               2 - dry land idetermined from iwetdry.c 
                                             ! 1024                           11 - standard deviation out of range (e.g. LRM 0.3 to 0.8)
                                             ! 2048                           12 - drange is out of bounds 
                                             ! 4096                           13 - swh is out of range (e.g. LRM 1 to 10 m)
                                             ! 8192                           14 - retrack failed least squares
                                             ! 16384                          15 - amp out of range (e.g. LRM 50000 to 80000)

        integer(kind=4) ::     csum          ! computed sum of counts of power for all range gates 
        integer(kind=4) ::     lon           ! units 10^-6 deg, 0 to 360 
        integer(kind=4) ::     lat           ! units 10^-6 deg  
        integer(kind=4) ::     mss           ! model mean sea surface, mm EGM2008 
        integer(kind=4) ::     esf_A         ! echo scale factor A 
        integer(kind=4) ::     esf_B         ! echo scale factor B - echo_power in watts = waveform_counts*(A*10^-9)*2^B 
        integer(kind=4) ::     pamp(2)       ! waveform power  estimated by retracking  
        integer(kind=4) ::     beam_dir(3)   ! beam direction vector in micrometers 
        integer(kind=4) ::     baseline(3)   ! interferometer baseline 
        integer(kind=4) ::     drange(3)     ! range correction from retracking, mm
        integer(kind=4) ::     beam(7);      ! std, mean, amp, skew, kurt, std of angle, stack centre angle 

        integer(kind=2) ::     surf_flag     ! surface flag type: 0-ocean; 1-closed sea; 2-continental ice; 3-land
        integer(kind=2) ::     pswh(2)       ! pseudo-swh, waveform spread estimated by retracking, mm  
        integer(kind=2) ::     rchisq(2)     ! root sum of squares of misfit error from tracker  
        integer(kind=2) ::     pnoise(2)     ! noise level preceding leading edge, est by retracking  
        integer(kind=2) ::     n_echo        ! number of echoes averaged into this record
        integer(kind=2) ::     echo_flag     ! bits 15-8:   approximate beam steering

        integer(kind=2) ::     decay(2)      ! plateau decay rate est by retrack, prop to xi  
        integer(kind=2) ::     hotide        ! (pure, only) ocean tide height, mm. 
        integer(kind=2) ::     hlotide       ! long period ocean tide height, mm.
        integer(kind=2) ::     hltide        ! load tide height, mm.
        integer(kind=2) ::     hstide        ! solid earth tide, mm. 
        integer(kind=2) ::     hptide        ! height correction for pole tide, mm.  From model.  
        integer(kind=2) ::     hionomod      ! height correction for iono delay, mm.  From a model.  
        integer(kind=2) ::     hionogim      ! height correction for iono delay, mm.  From JPL GIM.  
        integer(kind=2) ::     hdynatm       ! dynamic atmospheric correction, mm. From a model.
        integer(kind=2) ::     hwet          ! height correction for wet tropo delay, mm.  From a model.  
        integer(kind=2) ::     hdry          ! height correction for dry tropo delay, mm.  From a model.  
        integer(kind=2) ::     hinvb         ! height corr for inverse barometer, mm.  From model  
        integer(kind=2) ::     hdopp         ! height correction for doppler effect, radial component, mm. 
        integer(kind=2) ::     new_tide      ! new ocean tide model from CSR4.0, mm 

        integer(kind=2) ::     wave(512)     ! Ku-band waveform data.  
       end type reaa

