%
%   script to simulate a SAR waveform from CryoSat
%
clear
c = 2.998e08;   % speed of light
H=717000;       % spacacraft height
dt=2.86e-9;     % pulse length = 1/bandwidth
sig=328000;        % one sigma for antenna beam pattern
sig=328;        % one sigma for antenna beam pattern
%
% loop over time to compute the radius of the footprint
%
ig=-32:.1:64;
t=ig*dt;
t1=max(t,0.);
r1=sqrt(2.*c*H*t1);
t2=max(0,t-dt);
r2=sqrt(2.*c*H*t2);
%
%  integrate over the area
%
nw=length(r1);
dr1=diff(r1);
dr2=diff(r2);
for k=2:nw,
wave1(k)=area(r1(1:k),sig);
wave2(k)=area(r2(1:k),sig);
wave=wave1-wave2;
end
%
%  plot the waveform
%
figure(1)
clf
subplot(3,1,1),plot(ig,wave,'.');xlabel('gate #'),ylabel('area (m^2)')
%
%   convolve with a wave height distribution 
%
swh = 3.0;
sigw = swh/4./.468257;
sigw2 = 2.*sigw*sigw;
ig2=-32:.1:32;
t2=ig2.*ig2;
filt=exp(-t2/sigw2);
filt=filt/sum(filt);
subplot(3,1,2),plot(ig2,filt,'.');
%
waveswh=conv(wave,filt,'same');
subplot(3,1,3),plot(ig,waveswh);xlabel('gate #'),ylabel('area (m^2)')
%
%   plot them together
%
figure(2)
plot(ig,wave,ig,waveswh);xlabel('gate #'),ylabel('area (m^2)')
