/*  c20_select.c  lon0 lonf lat0 latf isat  < input.c20 > output.c20

program to window data in lat/lon space

  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "geosat10hz.h"
#include "rea.h"
#include "envisat18hz.h"
#include "cryosat20hz.h"
#include "jason20hz.h"

int main (int argc, char **argv) {

	struct GEOSAT10HZ    g;
	struct REA           r;
        struct ENVISAT18HZ   e;
	struct CRYOSAT20HZ c;
	struct CRYOSIN20HZ s;
	struct JASON20HZ j;
	int     nstr, kk, count, isat;
        double lon, lon0, lonf, lat, lat0, latf;
        FILE *fpin, *fpout;
	
/* Make sure usage is correct */
	if (argc != 8 ) { 
		fprintf (stderr, "usage:  c20_select  input.c20 output.c20 lon0 lonf lat0 latf isat \n");
                fprintf (stderr, "        lon0 lonf starting and ending longitude 0 - 360 \n");
                fprintf (stderr, "        lat0 latf starting and ending latitude -88 to 88 \n");
                fprintf (stderr, "        isat (1)-geosat, (2)-ers1, (3)-envisat, (4)-cryosat LRM and SAR, (5)-cryosat SIN, (6)-jason1 \n");
		exit (-1);
	}
        lon0 = atof(argv[3]);
        lonf = atof(argv[4]);
        lat0 = atof(argv[5]);
        latf = atof(argv[6]);
        isat = atoi(argv[7]);

/* check lon and lat for proper bounds */
        if(lon0 < 0. || lonf < 0. || lon0 > 360. || lonf > 360. || lon0 >= lonf) {
                fprintf (stderr," longitudes out of range %f %f \n", lon0, lonf);
                exit(-1);
        }
        if(lat0 < -89. || latf < -89. || lat0 > 89. || latf > 89. || lat0 >= latf) {
                fprintf (stderr," latitudes out of range %f %f \n", lat0, latf);
                exit(-1);
        }

/* open the input and output files */
        if((fpin = fopen(argv[1], "r")) == NULL){
                fprintf(stderr,"input file not opened \n");
                exit(-1);
        }
        if((fpout = fopen(argv[2], "w")) == NULL){
                fprintf(stderr,"output file not opened \n");
                exit(-1);
        }

/* select the data */
	if (isat == 1) {
	while ( (fread ((void *)&g, sizeof(struct GEOSAT10HZ), (size_t)1, fpin)) == 1 ) {
             lon=g.lon*1.e-7;
             if(lon < 0.) lon = lon +360.;
             lat=g.lat*1.e-7;
             if(lon >= lon0 && lon <= lonf && lat >= lat0 && lat <= latf) {
                 fwrite((void *)&g, sizeof(struct GEOSAT10HZ), (size_t)1, fpout);
             }
        }
	}
	else if (isat == 2) {
	while ( (fread ((void *)&r, sizeof(struct REA), (size_t)1, fpin)) == 1 ) {
             lon=r.lon*1.e-6;
             lat=r.lat*1.e-6;
             if(lon >= lon0 && lon <= lonf && lat >= lat0 && lat <= latf) {
                 fwrite((void *)&r, sizeof(struct REA), (size_t)1, fpout);
             }
        }
	}
	else if (isat == 3) {
	while ( (fread ((void *)&e, sizeof(struct ENVISAT18HZ), (size_t)1, fpin)) == 1 ) {
             lon=e.lon*1.e-6;
             lat=e.lat*1.e-6;
             if(lon >= lon0 && lon <= lonf && lat >= lat0 && lat <= latf) {
                 fwrite((void *)&e, sizeof(struct ENVISAT18HZ), (size_t)1, fpout);
             }
        }
        }
	else if (isat == 4) {
	while ( (fread ((void *)&c, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) == 1 ) {
             lon=c.lon*1.e-6;
             lat=c.lat*1.e-6;
             if(lon >= lon0 && lon <= lonf && lat >= lat0 && lat <= latf) {
                 fwrite((void *)&c, sizeof(struct CRYOSAT20HZ), (size_t)1, fpout);
             }
        }
        }
	else if (isat == 5) {
        while ( (fread ((void *)&s, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1 ) {
             lon=s.lon*1.e-6;
             lat=s.lat*1.e-6;
             if(lon >= lon0 && lon <= lonf && lat >= lat0 && lat <= latf) {
                 fwrite((void *)&s, sizeof(struct CRYOSIN20HZ), (size_t)1, fpout);
             }
        }
        }
	else if (isat == 6) {
        while ( (fread ((void *)&j, sizeof(struct JASON20HZ), (size_t)1, fpin)) == 1 ) {
             lon=j.lon*1.e-6;
             lat=j.lat*1.e-6;
             if(lon >= lon0 && lon <= lonf && lat >= lat0 && lat <= latf) {
                 fwrite((void *)&j, sizeof(struct JASON20HZ), (size_t)1, fpout);
             }
        }
        }

	else {
		fprintf(stderr," isat incorrect = %d \n",isat);
                exit(-1);
        }

	exit (0);
}
