/*  c20_dump.c input.c20 > output

program to read a Cryosat 20 Hz
and dump whatever.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSAT20HZ c;
	FILE	*fpin;
	int	i, n, k, tide, delay, h, h0, h1, h2,issb, kk, ioff, boff, nstr1, nstr2, tide_csr;
    	int     ngood=0, nbad=0;
	unsigned int flag;
	double	t,lat, lon, t0, alt, range, ssh, diff, wfac, sfac=0., swdiff, damp;
        double  rhtd, rhd1, rmean, rm2, rms;
        double  rht[20];
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 4 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_sinlp input.c20 nstruct baseline  \n");
        	fprintf (stderr, "        nstruct (1) - DO NOT apply corrections and tides \n");
        	fprintf (stderr, "        nstruct (2) - apply corrections and tides WITHOUT EDITING \n");
		fprintf (stderr, "        nstruct (3) - apply corrections and tides WITH EDITING \n");
		fprintf (stderr, " \n");
		fprintf (stderr, "        baseline (1) - Baseline A \n");
		fprintf (stderr, "        baseline (2) - Baseline B \n");
		exit (-1);
	}
    	nstr1 = atoi(argv[2]);
	nstr2 = atoi(argv[3]);
	k = 0;
	n = 0;
		
	while ( (fread ((void *)&c, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) == 1 ) {
		if(k == 0) t0 = 1.0e-06 * c.microsec + c.sec2000;
        k=k+1;
	/* t = 1.0e-06 * c.microsec + c.sec2000; */
	lat = 1.0e-06 * c.lat;
	lon = 1.0e-06 * c.lon;
	tide = c.hltide + c.hstide + c.hptide + c.hotide;
    	tide_csr = c.new_tide + c.hstide;
	/* don't use ionosphere for envisat */
	delay = c.hwet + c.hdry + c.hiono + c.hinvb;
   	ioff = -3440.0; /* based on analysis of ocean passes */
    	boff = -18728.0; /* to be determined from offset of Baseline A and B track points */
	/* issb = floor(0.060*c.pswh[1]+.5); don't use issb for land! */
	flag = 0;	
	switch ( nstr1 ) {
		case 1:
			h=c.alt-(c.range+ioff)-c.drange[2];
			break;
		case 2:
			h=c.alt-(c.range+c.hdopp+tide+delay+ioff)-c.drange[2];
			break;
		case 3:
			if (abs(c.hinvb) < 1 | abs(c.hdry) < 1 | abs(c.hwet) < 1 ) {
				flag = 1;	
			}
			h=c.alt-(c.range+c.hdopp+tide+delay+ioff)-c.drange[2];
			break;			
	}

	if ( nstr2 == 2 ) {
		h=h+boff;
	}
                
	if ( (flag & 1) == 0) { 
		printf("%lf %lf %d \n",lon,lat,h); 
	}
	}
	fclose (fpin);
	exit (0);
}
