!
      program ers20_cdr
!
!          Copyright 2001
!	   David T. Sandwell
!
!  program to read ers altimeter data in rea format.
!  the files are reformatted into binary direct access files.
!  the 20 per second data are low-pass filtered and then
!  compressed into 5/sec data.
!
!  1. bad ers data are edited for the following reasons:
!      land data
!      flag    flag turned on
!      swh     exceeds 8.0 m or less than 0.5 m
!      amp     exceeds 800  or less than 520 
!      sig0    exceeds 19.6 dB or is less than 7dB
!      dh      deviation from ocean geoid exceeds 1.5 m
!
!  2. reading continues until data time points are
!     separated by more than 4 seconds and/or delta
!     theta between successive points switches sign.
!     these conditions define a new pass.
!
!  3. five data points are created for each second
!     by filtering adjacent points from the 20/sec data.
!     slopes between data adjacent points and computed
!     using a first difference formula and stored with
!     the second time point. the very first slope in the array
!     is assigned a value the # of
!     points in the pass.   this value signifies the
!     begining of a pass.
!
!  4. the following corrections are summed and added to the height data.
!      solid earth tide
!      ocean tide
!      wet troposphere fnoc
!      dry troposphere
!      ionosphere
!
!  5.  The significant wave height is stored in the last open
!      4-byte area in the cdr. 
!      A 7.8% SSB is applied to the height.
!
!  6. each pass with more than 20 points is then dumped
!     into an ascending or descending binary file.
!
!     external funcs
      parameter (ND=60000,MP=1000,NP=50)
      implicit real*8 (d)

include "ea_structs.inc"

      type(rrea) rea
      common /iemas/irec(7*17000)
      common /info/ lcrt(2),lin(10),nin,lout(10),nout
      common /count/ncount(2)
      common /emas/ dtm(17000),rlt(17000),rln(17000),rsh(17000),rdsh(17000),rfsh(17000),rcor(17000)
      real*8  dm(ND),drt(ND),drn(ND)
      real*4  rh(ND),rswh(ND),rhout(ND),rswhs(ND)
      real*4  rswh0(ND),rsig0(ND)
      real*8  dflt(99)
      real*8 dmjd, tlp, tds, u(3,2), v(3,2) 
      logical isdata_g,isdata,pseu,radi
!
!  filter - 99 point, Park-McClellan low-pass filter
!
      data dflt(01)/ 4.8032623643126802d-05/
      data dflt(02)/ 7.3276464076935471d-05/
      data dflt(03)/ 1.1767854949285006d-04/
      data dflt(04)/ 1.6691557284204521d-04/
      data dflt(05)/ 2.1265942299358124d-04/
      data dflt(06)/ 2.4316901479037306d-04/
      data dflt(07)/ 2.4430402611052106d-04/
      data dflt(08)/ 2.0138426924749386d-04/
      data dflt(09)/ 1.0175481502884287d-04/
      data dflt(10)/-6.2042519379727309d-05/
      data dflt(11)/-2.8897186550265924d-04/
      data dflt(12)/-5.6672719475995648d-04/
      data dflt(13)/-8.7002637003286707d-04/
      data dflt(14)/-1.1604698114109134d-03/
      data dflt(15)/-1.3886661577988321d-03/
      data dflt(16)/-1.4989013619730617d-03/
      data dflt(17)/-1.4358167191815426d-03/
      data dflt(18)/-1.1530456217170618d-03/
      data dflt(19)/-6.2296691840840572d-04/
      data dflt(20)/ 1.5437542947298533d-04/
      data dflt(21)/ 1.1435127392807256d-03/
      data dflt(22)/ 2.2697323012373970d-03/
      data dflt(23)/ 3.4194384257467837d-03/
      data dflt(24)/ 4.4463830498333145d-03/
      data dflt(25)/ 5.1836983787819767d-03/
      data dflt(26)/ 5.4613946695376446d-03/
      data dflt(27)/ 5.1280629865201694d-03/
      data dflt(28)/ 4.0747290551395058d-03/
      data dflt(29)/ 2.2581413580114477d-03/
      data dflt(30)/-2.7960793149027426d-04/
      data dflt(31)/-3.3980425040013229d-03/
      data dflt(32)/-6.8546034531323692d-03/
      data dflt(33)/-1.0312135716136249d-02/
      data dflt(34)/-1.3358173618519813d-02/
      data dflt(35)/-1.5535586444661822d-02/
      data dflt(36)/-1.6382372895693168d-02/
      data dflt(37)/-1.5477285778228813d-02/
      data dflt(38)/-1.2487002213156954d-02/
      data dflt(39)/-7.2095476257611158d-03/
      data dflt(40)/ 3.9071184916639640d-04/
      data dflt(41)/ 1.0161326568078589d-02/
      data dflt(42)/ 2.1756466681306454d-02/
      data dflt(43)/ 3.4649960904781708d-02/
      data dflt(44)/ 4.8166535340459789d-02/
      data dflt(45)/ 6.1529179145275587d-02/
      data dflt(46)/ 7.3918741036108390d-02/
      data dflt(47)/ 8.4540254671122098d-02/
      data dflt(48)/ 9.2689784792296823d-02/
      data dflt(49)/ 9.7815042072190633d-02/
      data dflt(50)/ 9.9563789295822616d-02/
      data dflt(51)/ 9.7815042072190633d-02/
      data dflt(52)/ 9.2689784792296823d-02/
      data dflt(53)/ 8.4540254671122098d-02/
      data dflt(54)/ 7.3918741036108390d-02/
      data dflt(55)/ 6.1529179145275587d-02/
      data dflt(56)/ 4.8166535340459789d-02/
      data dflt(57)/ 3.4649960904781708d-02/
      data dflt(58)/ 2.1756466681306454d-02/
      data dflt(59)/ 1.0161326568078589d-02/
      data dflt(60)/ 3.9071184916639640d-04/
      data dflt(61)/-7.2095476257611158d-03/
      data dflt(62)/-1.2487002213156954d-02/
      data dflt(63)/-1.5477285778228813d-02/
      data dflt(64)/-1.6382372895693168d-02/
      data dflt(65)/-1.5535586444661822d-02/
      data dflt(66)/-1.3358173618519813d-02/
      data dflt(67)/-1.0312135716136249d-02/
      data dflt(68)/-6.8546034531323692d-03/
      data dflt(69)/-3.3980425040013229d-03/
      data dflt(70)/-2.7960793149027426d-04/
      data dflt(71)/ 2.2581413580114477d-03/
      data dflt(72)/ 4.0747290551395058d-03/
      data dflt(73)/ 5.1280629865201694d-03/
      data dflt(74)/ 5.4613946695376446d-03/
      data dflt(75)/ 5.1836983787819767d-03/
      data dflt(76)/ 4.4463830498333145d-03/
      data dflt(77)/ 3.4194384257467837d-03/
      data dflt(78)/ 2.2697323012373970d-03/
      data dflt(79)/ 1.1435127392807256d-03/
      data dflt(80)/ 1.5437542947298533d-04/
      data dflt(81)/-6.2296691840840572d-04/
      data dflt(82)/-1.1530456217170618d-03/
      data dflt(83)/-1.4358167191815426d-03/
      data dflt(84)/-1.4989013619730617d-03/
      data dflt(85)/-1.3886661577988321d-03/
      data dflt(86)/-1.1604698114109134d-03/
      data dflt(87)/-8.7002637003286707d-04/
      data dflt(88)/-5.6672719475995648d-04/
      data dflt(89)/-2.8897186550265924d-04/
      data dflt(90)/-6.2042519379727309d-05/
      data dflt(91)/ 1.0175481502884287d-04/
      data dflt(92)/ 2.0138426924749386d-04/
      data dflt(93)/ 2.4430402611052106d-04/
      data dflt(94)/ 2.4316901479037306d-04/
      data dflt(95)/ 2.1265942299358124d-04/
      data dflt(96)/ 1.6691557284204521d-04/
      data dflt(97)/ 1.1767854949285006d-04/
      data dflt(98)/ 7.3276464076935471d-05/
      data dflt(99)/ 4.8032623643126802d-05/
!
!  constants
!
      data td/.980400/
      data rad/.01745329e0/
!
!   open the CSR tide model
!
      open(51, file='/mgg/alt/grids/ortho_csr_4.0')
!
!  initialize counters
!
      ncount(1)=1
      ncount(2)=1
!
!  open binary input file for reading
!
      call inioc(lu)
!
!  open ascending and descending output files
!
      write(lcrt(2),901)
 901  format(' ascending, ',$)
      call outioc(lu1)
      write(lcrt(2),902)
 902  format(' descending, ',$)
      call outioc(lu2)
!
!  should the file header be written?
!
      write(lcrt(2),904)
 904  format(' write file header (1-yes) (0-no):  ',$)
      read(lcrt(1),*)iwrthead
!
!  the type is ers
!
      ityp=5
!
!   output files are Sun integers
!
      ifile=1
!
!  time increment is 5 Hz.
!
      idtt=1.d06*td/5.
      if(iwrthead.eq.1) then
!
!  write the satellite information to the output files
!
      iasc=1
      call writhead(lu1,ityp,iasc,ifile,idtt)
      iasc=2
      ncount(2)=1
      call writhead(lu2,ityp,iasc,ifile,idtt)
! 
!  reset both counters to  the second record
!
      ncount(1)=2
      ncount(2)=2
      endif
! 
! initialize other counters
!
      mdata=0
      nflag=0
      nswh=0
      namp=0
      nsig0=0
      ndev=0
      nland=0
!
!   offset based on fit to EGM2008
!
      ioff=145
!
! read a pass
!
  15  continue
      ip=1
!
      if(mdata.eq.0)then
      ip=0
      endif
!
!***********load the pass************************
!
  20  continue
!
!  read a 184-byte record.
!
      call rddiscb(lu,rea,184,istin)
      if(istin.lt.0) go to 50
!  
!  get the variabiles used for editing
!
      iflag=rea%flag
      swh=rea%swh/1000.
      amp=float(rea%amp)
      sig0=rea%sig0/100.
!
!  edit if one of the following flags is on
!  15756 = 4+8+128+256+1024+2048+4096+8192
!
      if(and(iflag,15756).ne.0) then
      nflag=nflag+1
      go to 20
      endif
!
!   check for dry area
!
      dlat=rea%lat*1.e-06
      dlon=rea%lon*1.e-06
      idry=iwetdry(dlat,dlon)
!
!  edit of not water
!
      if(idry.eq.2) then
      nland=nland+1
      go to 20
      endif
!
!  edit if swh exceeds 8.0 m or is less than 0.5 m
!
      if(swh.gt.8.0.or.swh.lt.0.5) then
      nswh=nswh+1
      go to 20
      endif
!
!  edit if amp exceeds 800 or is less than 520
!
      if(amp.gt.800.or.amp.lt.520) then
      namp=namp+1
      go to 20
      endif
!
!
!  edit if sig0 exceeds 19.6 or is less than 7
!
      if(sig0.gt.19.6.or.sig0.lt.7.0) then
      nsig0=nsig0+1
      go to 20
      endif
!
!   check for absurd height on ocean data only
!
!     rth=(rea%alt-(rea%range+rea%drange)-rea%tide+rea%corr)*1.e-3
!     dh=rth-rea%geoid*1.e-3+0.06*rea%swh*1.e-03+.57
!     if(abs(dh).gt.1.5.and.iflag.eq.0) then
!     ndev=ndev+1
!     go to 20
!     endif
!
! keep the record
!
      mdata=mdata+1
      ip=ip+1
!
!  decode a record
!
      dm(ip)=rea%utcs+rea%utcms*1.d-06
      drt(ip)=rea%lat*1.e-06
      drn(ip)=rea%lon*1.e-06
      rswh(ip)=rea%swh*1.e-3
!
!   apply all of the corrections and tide
!   this is the tital csr tide + the solid earth tide
!
      ddtide = rea%tide
!
!  compute a new tide from GOT and use if available
!
      dmjd = 46066.d0 + dm(ip)/86400.d0
      call csrtide (dmjd,drt(ip),drn(ip),dcsr,dtlp,dtds,isdata,u,v,pseu,radi)
!
! isolate the solid earth tide
!
      dcor6=ddtide-dcsr
!
! compute the got tide and add the solid earth component
!
      call perth3(drt(ip),drn(ip),dmjd,dtide,isdata_g)
!     if(isdata_g.and.100*(ip/100).eq.ip) write(*,*)ddtide,dcor6+dtide*10.,dcor6
      if(isdata_g) ddtide = dcor6+dtide*10.
      rh(ip)=(ioff+rea%alt-(rea%range+rea%drange)-ddtide+rea%corr)*1.e-3
!
!  apply a 7.8% SSB based on fit to EGM2008
!
      rh(ip)=rh(ip)+0.078*rswh(ip)
!
!   subtract the SSH model for evaluation purposes
!
!     rh(ip)=rh(ip)-rea%geoid*1.e-3
!
!
!  start filling the temp arrays
!
      if(ip.eq.1) go to 20
!
!  check for too many points
!
      if(ip.gt.ND) then
      write(lcrt(2),905)
 905  format(' warning ! # of points exceeded ')
      endif
!
!  check for change in pass orientation
!
      if(ip.eq.2)iasc0=sign(1.D0,drt(2)-drt(1))
      iasc=sign(1.D0,drt(ip)-drt(ip-1))
      if(iasc.ne.iasc0) go to 50
!
!  check for significant time gap
!
      ddtm=dm(ip)-dm(ip-1)
      if(ddtm.gt.4.d0.or.ddtm.le.0.d0) go to 50
      go to 20
!
!*********filter and write the pass**************************
!
   50 continue
!
!  omit short passes < 80 points
!
      ipm1=ip-1
      if(ipm1.le.80) go to 70
!
!  filter the 20/s data
!
      ik=0
      do 60 k=1,ipm1,4
      dsum=0.
      dsumw=0.
      do 55 l=-49,49
      kl=k+l
      kl=max(kl,1)
      kl=min(kl,ipm1)
      dsumw=dsumw+dflt(l+50)
      dsum=dsum+dflt(l+50)*rh(kl)
   55 continue
      if(dsumw.gt.0.8) then
        rhout(k)=dsum/dsumw
      else
        rhout(k)=-999.
      endif
!     write(*,903)rh(k),rhout(k)
!903  format(3f8.4)
!
!  dump the filtered data at 5 Hz
!
      if(rhout(k).lt.-990.) go to 60
      ik=ik+1
      dtm(ik)=dm(k)
      rlt(ik)=drt(k)
      rln(ik)=drn(k)
      rsh(ik)=rhout(k)
      rcor(ik)=rswh(k)
      if(ik.gt.1) then
         ddtm=dtm(ik)-dtm(ik-1)
         rdsh(ik)=(rsh(ik)-rsh(ik-1))/ddtm
      endif
 60   continue
!
!  the output array is now filled.
!  put the real data into an integer array and
!  write out the pass on the appropriate file.
!
      ik1=ik
      call realint(1,1,ik1)
      irec(6)=ik1
      if(iasc0.eq.1) then
        lott=lu1
        indx=1
      else if(iasc0.eq.-1) then
        lott=lu2
        indx=2
      else
        write(*,'(a)') '***** error iasc0'
        stop
      endif
      if(ik1.le.20) go to 70
      call wrdisc(lott,irec,7*ik1)
      ncount(indx)=ncount(indx)+ik1
!     write(lcrt(2),907)ik1,iasc0
! 907 format('  # of points,  ascending(1) or descending(-1) ; ',2i6)
!
!  stuff the last record read into the first slot.
!
  70  continue
      if(istin.lt.0) go to 999
      dm(1)=dm(ip)
      drt(1)=drt(ip)
      drn(1)=drn(ip)
      rswh(1)=rswh(ip)
      rh(1)=rh(ip)
      go to 15
!
!  close the files.
!
 999  continue
      call closio(-1)
      write(lcrt(2),912)nflag,nswh,namp,nsig0,ndev,nland
 912  format(/,'edited: flag, swh, amp, sig0, dev, land = ',6i9)
      write(lcrt(2),911)mdata
 911  format(i12, ' data points not edited ')
      nasc=ncount(1)-1
      ndes=ncount(2)-1
      write(lcrt(2),913)nasc,ndes
 913  format(' # ascending file, # descending file ',2i12)
      stop
      end
!
!
      block data luio
!
!  common blocks for logical unit # transfers.
!
      common/info/lcrt(2),lin(10),nin,lout(10),nout
!
!   lcrt(1)  -  lu for terminal read.
!   lcrt(2)  -  lu for terminal write.
!   lin(i)   -  i'th input file lu (10-19 only).
!   nin      -  # of input files opened.
!   lout(i)  -  i'th output file lu (20-29 only).
!   nout     -  # of output files opened.
!
!***** calls no other routines
!
      data lcrt/5,6/
      data lin,nin/10*5,0/
      data lout,nout/10*6,0/
      end

