!==============================================================================
!       G10.INC  Structure definitions for Geosat waveform record
!==============================================================================
!

        type gg10

	integer(kind=4) :: kframe	! Frame counter.
					! This is a unique integer that increases by 1
					! at each 0.098... second waveform value.
					! kframe = 32*(Major frame count) + (Minor frame count).
					! Combines items 1 and 2 of Table 2 or 5.  
	
	integer(kind=4) :: tai85s	! atomic seconds since 1985  
	integer(kind=4) :: nanos	! nanoseconds since tai85s  (See NOTE ONE of sdrwdr.h) 
	integer(kind=4) :: alt		! orbit altitude, mm  (SEE NOTE ONE of sdrwdr.h) 
	integer(kind=4) :: range	! range, per on-board tracker, mm (item 5 thru 14) 
	integer(kind=4) :: modebits	! "mode" word; (item 4, only least-significant 30 bits used) 
	integer(kind=4) :: flagbits	! "flagword";  (item 5, only least-significant 30 bits used) 
	integer(kind=4) :: trackbits	! Useful flags to be defined as we go along; initially zero  
					! 1  - waveform unavailable
					! 2  - failed fibtrack5 or 4, retracking
                                        ! 4  - dry land
					! 8  - pnoise < 0 or pnoise > 420
					! 16 - pagc < 700 or pagc > 2200
					! 32 - pswh < 100 or pswh > 14000
					! 64 - chisq(1) > 3000
					!128 - failed fibtrack3 or 1
					!256 - chisq(2) > 150  uses weight = (5 + power)/10
					!512 - decay(2) > 5600 

	integer(kind=4) :: lon		! degrees * 10^6 or * 10^7 if -180 to 180  
	integer(kind=4) :: lat		! degrees same units as lon 
	integer(kind=4) :: mss		! model mean sea surface (CLS00.1 ?), mm  
	integer(kind=4) :: ispare	! spare not yet defined rounds this part to 48 bytes  
	
	integer(kind=2) :: pswh(2)	! pseudo-swh, waveform spread estimated by retracking, mm  
	integer(kind=2) :: pagc(2)	! pseudo-agc, waveform power  estimated by retracking  x 10
	integer(kind=2) :: rchisq(2)	! root sum of squares of misfit error from tracker x 100 
	integer(kind=2) :: pnoise(2)	! noise level preceding leading edge, est by retracking  x 10
	integer(kind=2) :: drange(2)	! range correction from retracking, mm  
	integer(kind=2) :: decay(2)	! plateau decay rate est by retrack, prop to xi  
	
	integer(kind=2) :: ppp		! pulse peakiness parameter, x1000  
	integer(kind=2) :: swh		! swh from on-board tracker, 0 to 2000 cm (item 18 thru 27) 
	integer(kind=2) :: agc		! agc (on-board), 0 to 6400 units of 0.01 dB (item 30 to 39)  
	integer(kind=2) :: xi		! off-nadir angle, 0 to 200 units of 0.01 degree.  (item 45)
					! This is derived from vatt and may not be useful. 
	integer(kind=2) :: sig0		! sigma-naught, 0 to 6400 units of 0.01 dB.  (item 15)  
					! This is computed from 1-Hz avg agc using a naive constant 
					! altitude.  May not be useful once we understand how to go
					! from agc to sig0.  
	integer(kind=2) :: wc		! Wind speed, 0 to 320 units of 0.1 m/s.  (item 47)
					! This is derived from sig0 and may not be useful to carry.  
	integer(kind=2) :: vatt_t	! voltage proportional to attitude, 0 to 3000 millivolts.  (item 48)
					! Derived including cal correction but not 60-sec smoother?  

	integer(kind=2) :: dhsx		! height correction from (SWH|xi) lookup, +/-9999 mm (item 16) 
	integer(kind=2) :: dhfm		! height correction from pulse fm error, +/-999 mm (item 17) 
	integer(kind=2) :: dswhsx	! swh correction from (SWH|xi) lookup, +/-999 cm  (item 29) 
	integer(kind=2) :: dagcsx	! agc correction from (SWH|xi) lookup, +/-999 units of 0.01 dB (item 41)  
	integer(kind=2) :: dagch	! agc correction for height, +/-999 units of 0.01 dB (item 42)  
	integer(kind=2) :: dagct	! agc correction for temperature, +/-999 units of 0.01 dB (item 43)  
	integer(kind=2) :: rcvt0	! receiver temperature, -300 to +1100 units of 0.1 degrees C (item 49)  
	integer(kind=2) :: dhdt		! dh/dt estimate from on-board range rate, -99 to +99 m/s (item 44)  
	
	integer(kind=2) :: hotide	! total ocean tide from CSR4.0 csr40tide_(&time85, &lat, &lon, &tide)
	integer(kind=2) :: hltide	! load tide height, mm.  From model (FES2002?).  
	integer(kind=2) :: hstide	! solid earth tide, mm.  From model.  
	integer(kind=2) :: hptide	! height correction for pole tide, mm.  From model.  
	integer(kind=2) :: hiono	! height correction for iono delay, mm.  From a model.  
	integer(kind=2) :: hwet		! height correction for wet tropo delay, mm.  From a model.  
	integer(kind=2) :: hdry		! height correction for dry tropo delay, mm.  From a model.  
	integer(kind=2) :: hinvb	! height corr for inverse barometer, mm.  From model  
						
	integer(kind=2) :: hssb		! height corr for sea state bias, from a model (to be fit ?).  
	integer(kind=2) :: pdopp2	! Not given.  Space to store a home-grown doppler correction
					! to range.  The ICD claims that a 1-Hz avg of on-board-tracked
					! range-rate is sufficient for the doppler correction, but Chelton
					! et al (AP, 2001) indicate more smoothing needed due to noise.
					! Required is target range rate, not orbit height rate, because
					! geoid slope in trenches is significant at the mm level.  
	integer(kind=2) :: gindex	!  Not given in the SDR or WDR but perhaps guessable from
					! corrections depending on SWH and xi in ICD Table A-1.  (1 thru 5).
					! This may be useful for sorting out on-board tracker behavior,
					! and is a parameter needed to follow the recipes for waveform
					! calculations and corrections to on-board estimates in the ICD.  
	integer(kind=2) :: trackmode	! 1 thru 4 for a valid track mode, else zero  
	integer(kind=2) :: seq10	! 0 thru 9 indicating sequence number in 1-Hz frame  
	
	integer(kind=1) :: wave(64)	! waveform data.  First 60 locations are the 60-gate waveform.
					! Next three are the halfway-in-between fine-tracking gates.
					! (These 63 are stored consecutively in the same way in the WDR,
					! as an array(63,10) (fortran order) beginning as Item 5 of Table 5.)
					! Last location is the dividing factor (1,2 or 4) used to scale
					! waveform down to fit in unsigned char (done on board).
					! (These are stored as items 635 to 644 of Table 5)  
        end type gg10
