!
      program g10_cdr
!
!          Copyright 2004
!	   David T. Sandwell
!
!  program to read geosat altimeter data in G10 format.
!  the files are reformatted into binary direct access files.
!  the 10 per second data are low-pass filtered and then
!  compressed into 5/sec data.
!
!  1. bad geosat data are edited for the following reasons:
!         1  - waveform unavailable
!         2  - failed fibtrack, retracking
!         4  - dry land
!         8  - pnoise < 0 or pnoise > 420
!         16 - pagc < 700 or pagc > 2200
!         32 - pswh < 300 or pswh > 10000
!         64 - chisq(1) > 30
!        128 - failed fibtrack1
!        256 - chisq(2) > 1.5  uses weight = (5 + power)/10
!        512 - kappa > 5600
!
!  2. reading continues until data time points are
!     separated by more than 4 seconds and/or delta
!     theta between successive points switches sign.
!     these conditions define a new pass.
!
!  3. five data points are created for each second
!     by filtering adjacent points from the 10/sec data.
!     slopes between data adjacent points and computed
!     using a first difference formula and stored with
!     the second time point. the very first slope in the array
!     is assigned a value the # of
!     points in the pass.   this value signifies the
!     begining of a pass.
!
!  4. the following corrections are summed and added to the height data.
!      solid earth tide
!      ocean tide
!      wet troposphere fnoc
!      dry troposphere
!      ionosphere
!
!  5.  The significant wave height is stored in the last open
!      4-byte area in the cdr. 
!      A 6% SSB is applied to the height.
!
!  6. each pass with more than 20 points is then dumped
!     into an ascending or descending binary file.
!
      parameter (ND=60000,MP=1000,NP=50)
      implicit real*8 (d)

include "G10.inc"

      type(gg10) g10
      common /iemas/irec(7*17000)
      common /info/ lcrt(2),lin(10),nin,lout(10),nout
      common /count/ncount(2)
      common /emas/ dtm(17000),rlt(17000),rln(17000),rsh(17000),rdsh(17000),rfsh(17000),rcor(17000)
      real*8  dm(ND),drt(ND),drn(ND)
      real*4  rh(ND),rswh(ND),rhout(ND),rcr(ND)
      real*8  dflt(49),rhtt
      integer*4 itide,idelay
      logical isdata
!
!  filter - 49 point, Park-McClellan low-pass filter
!
      data dflt(01)/ 6.7992281670401002d-05/
      data dflt(02)/ 1.8700694870889918d-04/
      data dflt(03)/ 3.1528607642488873d-04/
      data dflt(04)/ 3.0861290212670162d-04/
      data dflt(05)/-1.7475937575698839d-05/
      data dflt(06)/-7.5518106817860497d-04/
      data dflt(07)/-1.7310039326269425d-03/
      data dflt(08)/-2.4124454761691970d-03/
      data dflt(09)/-2.0302616177715971d-03/
      data dflt(10)/ 2.9395930067600120d-05/
      data dflt(11)/ 3.6783622099612260d-03/
      data dflt(12)/ 7.7170107088505423d-03/
      data dflt(13)/ 9.9152608190486054d-03/
      data dflt(14)/ 7.7921828984917584d-03/
      data dflt(15)/-3.9212838026538676d-05/
      data dflt(16)/-1.2489583272410164d-02/
      data dflt(17)/-2.5324327480750111d-02/
      data dflt(18)/-3.1826114863849821d-02/
      data dflt(19)/-2.4891444267630965d-02/
      data dflt(20)/ 4.3178725840072582d-05/
      data dflt(21)/ 4.2407168820889546d-02/
      data dflt(22)/ 9.5478579738808494d-02/
      data dflt(23)/ 1.4767176310369906d-01/
      data dflt(24)/ 1.8591558283218612d-01/
      data dflt(25)/ 1.9995578552537749d-01/
      data dflt(26)/ 1.8591558283218612d-01/
      data dflt(27)/ 1.4767176310369906d-01/
      data dflt(28)/ 9.5478579738808494d-02/
      data dflt(29)/ 4.2407168820889546d-02/
      data dflt(30)/ 4.3178725840072582d-05/
      data dflt(31)/-2.4891444267630965d-02/
      data dflt(32)/-3.1826114863849821d-02/
      data dflt(33)/-2.5324327480750111d-02/
      data dflt(34)/-1.2489583272410164d-02/
      data dflt(35)/-3.9212838026538676d-05/
      data dflt(36)/ 7.7921828984917584d-03/
      data dflt(37)/ 9.9152608190486054d-03/
      data dflt(38)/ 7.7170107088505423d-03/
      data dflt(39)/ 3.6783622099612260d-03/
      data dflt(40)/ 2.9395930067600120d-05/
      data dflt(41)/-2.0302616177715971d-03/
      data dflt(42)/-2.4124454761691970d-03/
      data dflt(43)/-1.7310039326269425d-03/
      data dflt(44)/-7.5518106817860497d-04/
      data dflt(45)/-1.7475937575698839d-05/
      data dflt(46)/ 3.0861290212670162d-04/
      data dflt(47)/ 3.1528607642488873d-04/
      data dflt(48)/ 1.8700694870889918d-04/
      data dflt(49)/ 6.7992281670401002d-05/
!
!  constants
!
      data td/.9799210/
      data rad/.01745329e0/
!
!  initialize counters
!
      ncount(1)=1
      ncount(2)=1
!
!  open binary input file for reading 
!
      call inioc(lu)
!
!  open ascending and descending output files
!
      write(lcrt(2),901)
 901  format(' ascending, ',$)
      call outioc(lu1)
      write(lcrt(2),902)
 902  format(' descending, ',$)
      call outioc(lu2)
!
!  should the file header be written?
!
      write(lcrt(2),904)
 904  format(' write file header (1-yes) (0-no):  ',$)
      read(lcrt(1),*)iwrthead
!
!  the type is geosat
!
      ityp=3
!
!   output files are integers
!
      ifile=1
!
!  time increment is 5 Hz.
!
      idtt=1.d06*td/5.
      if(iwrthead.eq.1) then
!
!  write the satellite information to the output files
!
      iasc=1
      call writhead(lu1,ityp,iasc,ifile,idtt)
      iasc=2
      ncount(2)=1
      call writhead(lu2,ityp,iasc,ifile,idtt)
! 
!  reset both counters to  the second record
!
      ncount(1)=2
      ncount(2)=2
      endif
! 
! initialize other counters
!
      mdata=0
      nflag=0
      nswh=0
      ndev=0
      nland=0
!
!  offset based on fit to EGM2008
!
      ioff=-370
!
! read a pass
!
  15  continue
      ip=1
!
      if(mdata.eq.0)then
      ip=0
      endif
!
!***********load the pass************************
!
  20  continue
!
!  read a 192-byte record.
!
      call rddiscb(lu,g10,192,istin)
      if(istin.lt.0) go to 50
!  
!  get the variabiles used for editing
!
      iflag=g10%trackbits
      swh=g10%pswh(2)*1.e-03
!
!  edit if one of the following flags is on
!  1023 = 1+2+4+8+16+32+64+128+256+512
!  don't include the wetdry flag
!  1019 = 1+2+8+16+32+64+128+256+512
!
      if(and(iflag,1023).ne.0) then
      nflag=nflag+1
      go to 20
      endif
!
!   check for dry area
!
      dlat=g10%lat*1.d-07
      dlon=g10%lon*1.d-07
      idry=iwetdry(dlat,dlon)
!
!  edit of not water
!
      if(idry.eq.2) then
      nland=nland+1
      go to 20
      endif
!
!  edit if swh exceed 10.0 m or is less than 0.3 m
!
      if(swh.gt.10.0.or.swh.lt.0.3) then
      nswh=nswh+1
      go to 20
      endif
!
! keep the record
!
      mdata=mdata+1
      ip=ip+1
!
!  decode a record
!
      dm(ip)=g10%tai85s+g10%nanos*1.d-09
      drt(ip)=g10%lat*1.d-07
      drn(ip)=g10%lon*1.d-07 
      if(drn(ip).lt.0.d0) drn(ip)=drn(ip)+360.d0
      rswh(ip)=g10%pswh(2)*1.e-03
!
!   decode the tides, delays, doppler and CM offset
!
      itide0 =  g10%hstide + g10%hptide
      if (g10%hotide .ne. 32767) itide = itide0+g10%hotide
!
!  compute a new tide from GOT and use if available
!
      dmjd = 46066.d0 + dm(ip)/86400.d0
      call perth3(drt(ip),drn(ip),dmjd,dtide,isdata)
!     if(isdata.and.100*(ip/100).eq.ip) write(*,*)itide,itide0+dtide*10.
      if(isdata) itide = itide0+dtide*10.
!
      idelay = g10%hwet + g10%hdry + g10%hiono + g10%hinvb
      idopp = g10%pdopp2 +1795
      rcr(ip)=(itide+idelay)*1.e-03
!
!  contruct the height applying the drange corrections 
!
      rhtt=(ioff+g10%alt-g10%range-idopp-g10%drange(2))*1.e-03
      rh(ip)=rhtt-rcr(ip)
!
!  apply a 6.1% SSB based on fit to EGM2008
!
      rh(ip)=rh(ip)+0.061*rswh(ip)
!
!   subtract the SSH model for evaluation purposes
!
!     rh(ip)=rh(ip)-g10%mss*1.e-03
!
!
!  start filling the temp arrays
!
      if(ip.eq.1) go to 20
!
!  check for too many points
!
      if(ip.gt.ND) then
      write(lcrt(2),905)
 905  format(' warning ! # of points exceeded ')
      endif
!
!  check for change in pass orientation
!
      if(ip.eq.2)iasc0=sign(1.D0,drt(2)-drt(1))
      iasc=sign(1.D0,drt(ip)-drt(ip-1))
      if(iasc.ne.iasc0) go to 50
!
!  check for significant time gap
!
      ddtm=dm(ip)-dm(ip-1)
      if(ddtm.gt.4.d0.or.ddtm.le.0.d0) go to 50
      go to 20
!
!*********filter and write the pass**************************
!
   50 continue
!
!  omit short passes < 20 points
!
      ipm1=ip-1
      if(ipm1.le.20) go to 70
!
!  filter the 20/s data
!
      ik=0
      do 60 k=1,ipm1,2
      dsum=0.
      dsumw=0.
      do 55 l=-24,24
      kl=k+l
      kl=max(kl,1)
      kl=min(kl,ipm1)
      dsumw=dsumw+dflt(l+25)
      dsum=dsum+dflt(l+25)*rh(kl)
   55 continue
      if(dsumw.gt.0.8) then
        rhout(k)=dsum/dsumw
      else
        rhout(k)=-999.
      endif
!     write(*,903) dm(k),rh(k),rhout(k)
!903  format(f14.4,2f8.4)
!
!  dump the filtered data at 5 Hz
!
      if(rhout(k).lt.-990.) go to 60
      ik=ik+1
      dtm(ik)=dm(k)
      rlt(ik)=drt(k)
      rln(ik)=drn(k)
      rsh(ik)=rhout(k)
      rcor(ik)=rswh(k)
      if(ik.gt.1) then
         ddtm=dtm(ik)-dtm(ik-1)
         rdsh(ik)=(rsh(ik)-rsh(ik-1))/ddtm
      endif
 60   continue
!
!  the output array is now filled.
!  put the real data into an integer array and
!  write out the pass on the appropriate file.
!
      ik1=ik
      call realint(1,1,ik1)
      irec(6)=ik1
      if(iasc0.eq.1) then
        lott=lu1
        indx=1
      else if(iasc0.eq.-1) then
        lott=lu2
        indx=2
      else
        write(*,'(a)') '***** error iasc0'
        stop
      endif
      if(ik1.le.20) go to 70
      call wrdisc(lott,irec,7*ik1)
      ncount(indx)=ncount(indx)+ik1
!     write(lcrt(2),907)ik1,iasc0
! 907 format('  # of points,  ascending(1) or descending(-1) ; ',2i6)
!
!  stuff the last record read into the first slot.
!
  70  continue
      if(istin.lt.0) go to 999
      dm(1)=dm(ip)
      drt(1)=drt(ip)
      drn(1)=drn(ip)
      rswh(1)=rswh(ip)
      rh(1)=rh(ip)
      rcr(1)=rcr(ip)
      go to 15
!
!  close the files.
!
 999  continue
      call closio(-1)
      write(lcrt(2),912)nflag,nswh,nland
 912  format(/,'edited: flag, swh land  = ',3i9)
      write(lcrt(2),911)mdata
 911  format(i12, ' data points not edited ')
      nasc=ncount(1)-1
      ndes=ncount(2)-1
      write(lcrt(2),913)nasc,ndes
 913  format(' # ascending file, # descending file ',2i12)
      stop
      end
!
!
      block data luio
!
!  common blocks for logical unit # transfers.
!
      common/info/lcrt(2),lin(10),nin,lout(10),nout
!
!   lcrt(1)  -  lu for terminal read.
!   lcrt(2)  -  lu for terminal write.
!   lin(i)   -  i'th input file lu (10-19 only).
!   nin      -  # of input files opened.
!   lout(i)  -  i'th output file lu (20-29 only).
!   nout     -  # of output files opened.
!
!***** calls no other routines
!
      data lcrt/5,6/
      data lin,nin/10*5,0/
      data lout,nout/10*6,0/
      end

