/*  c20_dump.c input.c20 > output

program to read a Cryosat 20 Hz
and dump whatever.

  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSIN20HZ c;
	FILE	*fpin;
	int h_th,h_e3,h_e2;
	int     nstr, kk, count, iflag;
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 4 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_dump input.c20 nstruct trackbits \n");
                fprintf (stderr, "        nstruct (1)-int \n");
                fprintf (stderr, "        nstruct (2)-retrack results \n");
                fprintf (stderr, "        nstruct (3)-corrections \n");
                fprintf (stderr, "        nstruct (4)-waveform \n");
                fprintf (stderr, "        trackbits (0)-all data (1)-edit flagged data \n");
		exit (-1);
	}
        nstr = atoi(argv[2]);
        iflag = atoi(argv[3]);

	while ( (fread ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1 ) {

/* dump the the first part of the record */
	if( nstr == 1 && (c.trackbits == 0 || iflag == 0)) {
		printf(" sec2000   %d \n", c.sec2000);
		printf(" microsec  %d \n", c.microsec);
		printf(" kframe    %d \n", c.kframe);
		printf(" alt       %d \n", c.alt);
		printf(" alt_rate  %d \n", c.alt_rate);
		printf(" range     %d \n", c.range);
		printf(" range_s   %d \n", c.range_s);
		printf(" surf_flag %d \n", c.surf_flag);
		printf(" trackbits %d \n", c.trackbits);
		printf(" lon  %d \n", c.lon );
		printf(" lat  %d \n", c.lat );
		printf(" mss  %d \n", c.mss );
		printf(" esf_A %d \n",c.esf_A );
		printf(" esf_B %d \n",c.esf_B );
		printf(" beam_dir %d %d %d \n",c.beam_dir[0],c.beam_dir[1],c.beam_dir[2]);
		printf(" baseline %d %d %d \n",c.baseline[0],c.baseline[1],c.baseline[2]);
		printf(" beam_params std, mean, amp, skew, kurt %d %d %d %d %d \n",c.beam[0],c.beam[1],c.beam[2],c.beam[3],c.beam[4]);
		printf(" n_echo %d \n",c.n_echo);
        	printf("\n");
	}

/* dump the retrack results */
	if( nstr == 2 && (c.trackbits == 0 || iflag == 0)) {
		printf(" pamp[2]  %d %d \n", c.pamp[0],c.pamp[1] );
		printf(" pswh[2] %d %d \n", c.pswh[0],c.pswh[1] );
		printf(" rchisq[2] %d %d \n", c.rchisq[0],c.rchisq[1]);
		printf(" pnoise[2] %d %d \n", c.pnoise[0],c.pnoise[1]);
		printf(" drange[3] %d %d %d \n", c.drange[0],c.drange[1],c.drange[2] );
                h_e3=c.alt-(c.range+c.drange[0])-c.mss+3850;
                h_e2=c.alt-(c.range+c.drange[1])-c.mss+3850;
                h_th=c.alt-(c.range+c.drange[2])-c.mss+3850;
		printf(" height[3] %d %d %d \n",h_e3,h_e2,h_th);
		printf(" decay[2] %d %d \n", c.decay[0],c.decay[1]);
		printf(" csum %d \n", c.csum);
        	printf("\n");
	}

/* dump the corrections results */
	if( nstr == 3 && (c.trackbits == 0  || iflag == 0)) {
		printf(" hotide  %d \n", c.hotide );
		printf(" hltide  %d \n", c.hltide );
		printf(" hstide  %d \n", c.hstide );
		printf(" hptide  %d \n", c.hptide );
		printf(" hiono   %d \n", c.hionomod );
		printf(" hwet    %d \n", c.hwet );
		printf(" hdry    %d \n", c.hdry );
		printf(" hinvb   %d \n", c.hinvb );
		printf(" hdopp   %d \n", c.hdopp );
		printf(" new_tide %d \n", c.new_tide );
        	printf("\n");
	}

/* dump raw waveforms */
	if( nstr == 4 && (c.trackbits == 0 || iflag == 0)) {
           for (kk=0; kk<512; kk++){
            /*count=(int)(c.wave[kk]*(50.*65535)/(double)c.csum);*/
            count=(int)(c.wave[kk]);
	    printf(" %d ",count);
          }
       	printf("\n");
        }
	}
	fclose (fpin);
	exit (0);
}

