/*  c20_trkout input.c20 > output

program to read a Cryosat 20 Hz
and output latitude and longitude

  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSAT20HZ c;
	FILE	*fpin;
	int	i, n, k, tide, delay, h, h0, h1, h2,issb, kk, ioff, nstr, tide_csr;
	unsigned int itest;
    int     ngood=0, nbad=0;
	double	t,lat, lon, t0, alt, range, ssh, diff, wfac, sfac=0., swdiff, damp;
	double	latprev, lonprev, tnow, tprev;
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 3 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_hpr  input.c20 nstruct  \n");
                fprintf (stderr, "        nstruct (1)-use CryoSat on-board range \n");
                fprintf (stderr, "        nstruct (2)-use 2-parameter retracking result \n");
                fprintf (stderr, "        nstruct (3)-use 3-parameter retracking result \n");
                fprintf (stderr, "        nstruct (4)-use threshold retracking result \n");
		exit (-1);
	}
	nstr = atoi(argv[2]);
	k = 0;
    n = 0;
    tprev = 0.0;
    latprev = 0.0;
    lonprev = 0.0;
    
	while ( (fread ((void *)&c, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) == 1 ) {
        lat = 1.0e-06 * c.lat;
		lon = 1.0e-06 * c.lon;
        tnow = 1.0e-06*(double)c.microsec+(double)c.sec2000;
        
        tide = c.hltide + c.hstide + c.hptide + c.hotide;
        tide_csr = c.new_tide + c.hstide;
        delay = c.hwet + c.hdry + c.hiono + c.hinvb;
        ioff = -3800.0; /* based on analysis by Remko Scharroo at 2011 Crypsat meeting */
        issb = floor(0.060*c.pswh[1]+.5);
		h0 =c.alt-(c.range+c.hdopp+tide_csr+delay+ioff)+issb;
		swdiff = abs(c.pswh[1]-c.pswh[0]);
		
		switch ( nstr ) {
			case 1:
				h=h0;
				break;
			case 2:
				if (swdiff < 900) {
					h = h0 - c.drange[1];
				}
				else {
					itest = c.trackbits;
					if( (itest & 4096) == 0) {
						c.trackbits = c.trackbits + 4096; 
					}
				}
				break;
			case 3:
				h = h0 - c.drange[0];
				break;
			case 4:
				h = h0 - c.drange[2];
				break;
		}

		if(c.trackbits == 0) {
			/* detect breaks in passes 
			if ((k != 0) & (((tnow-tprev) > 1.0) | ((abs(lat-latprev)) > 0.003) | ((abs(lon-lonprev)) > 0.003)) ) {
				printf("> \n");
			} */	
			printf("%lf %lf %d %d \n",lon,lat,h,c.mss);
        	ngood = ngood +1;
        } 
        else {
        	nbad=nbad+1;
        }		
			
		tprev=tnow;
		latprev=lat;
		lonprev=lon;
		k++;
	}
	fclose (fpin);
	exit (0);
}
