/*  c20_dump.c input.c20 > output

program to read a Cryosat 20 Hz
and dump whatever you say below.

  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

main (int argc, char **argv) {

	struct CRYOSAT20HZ c;
	FILE	*fpin;
	int	i, n, k, tide, delay, h0, h1, issb, iedit, kk;
	double	t, t2000, lat, lon, t0;
        double  rhtd, rhd1, sum1, sum2, rmean, rm2, rms;
        double  rht[20];
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 2 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  c20_plot  input.c20  \n");
		exit (-1);
	}

/*  zero the 20 Hz array */	
        for (i = 0; i < 20; i++) {
          rht[i] = 0.;
        }
	k = 0;
        n = 0;
        sum1=0.;
        sum2=0.;
        iedit = 2000;
	while ( (fread ((void *)&c, sizeof(struct CRYOSAT20HZ), (size_t)1, fpin)) == 1 ) {

		if(k == 0) t0 = 1.0e-06 * c.microsec + c.sec2000;
                k=k+1;
		t2000 = 1.0e-06 * c.microsec + c.sec2000;
                t = t2000 - t0;
               /* if (t < 1.0e-6) printf("\n Error in index number %d", k);*/
                 
		lat = 1.0e-06 * c.lat;
		lon = 1.0e-06 * c.lon;
/*              printf("\n %f %f %f", t, lat, lon); */
		tide = c.hltide + c.hstide + c.hptide;
		if (c.hotide != 32767) tide = tide + c.hotide;
		delay = c.hwet + c.hdry + c.hiono + c.hinvb;
/*              printf("hwet + hdry + hiono + hinvb %d %d %d %d \n",c.hwet , c.hdry , c.hiono , c.hinvb); */
                issb=floor(0.060*c.pswh[1]+.5);
		h0 =(c.alt-(c.range+c.hdopp))-c.drange[0]-tide-delay+issb; 
		h1 =(c.alt-(c.range+c.hdopp))-c.drange[1]-tide-delay+issb; 
/*              printf(" %d %d %d %d \n",c.alt,c.range,c.hdopp,c.drange[1]); */
                rhd1=h1-c.mss; 
/*                printf(" %f %f %f %d %d %f \n", t, lat, lon, h1, c.mss, rhd1); */

/* calculate the rms mean and height */

                
		if(c.trackbits > 0 || c.rchisq[1] > iedit || fabs(rhd1) > 4000. || k < 10) continue;

                n=n+1;
                sum1=sum1+rhd1;
                sum2=sum2+rhd1*rhd1;

/* dump raw waveforms */
          for (kk=0; kk<128; kk++){
	    /*if(c.trackbits == 0) printf(" %d \n",c.wave[kk]);*/
	    /*printf(" %d \n",c.wave[kk]);*/
          }
	}
        rmean = sum1/n;
        rm2   = sum2/n;
        rms=sqrt(rm2-rmean*rmean);
        printf("%d %d %f %f \n",n,iedit,rmean,rms);
	fclose (fpin);
	exit (0);
}
