/*  c20_dump.c input.c20 > output

program to read a Cryosat 20 Hz
and dump whatever.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cryosat20hz.h"

int main (int argc, char **argv) {

	struct CRYOSIN20HZ c;
	FILE	*fpin;
	int	i, n, k, tide, delay, h, cor, h0, h1, h2,issb, kk, ioff, nstr, tide_csr;
    int     ngood=0, nbad=0;
	double	t,lat, lon, t0, alt, range, ssh, diff, wfac, sfac=0., swdiff, damp;
        double  rhtd, rhd1, rmean, rm2, rms;
        double  rht[20];
	
	/* Make sure usage is right by opening files  */
	
	if (argc != 3 || (fpin = fopen(argv[1], "r")) == NULL) {
		fprintf (stderr, "usage:  s20_cor input.c20 nstr \n");
		fprintf (stderr, "nstr (1) - wet troposphere delay \n");
		fprintf (stderr, "nstr (2) - dry troposphere delay \n");
		fprintf (stderr, "nstr (3) - inverse barometer effect \n");
		fprintf (stderr, "nstr (4) - doppler effect \n");
		fprintf (stderr, "nstr (5) - ionospheric delay from Bent model \n");
		fprintf (stderr, "nstr (6) - ionospheric delay from JPL GIM \n");
		exit (-1);
	}
    nstr = atoi(argv[2]);
	k = 0;
	n = 0;
		
	while ( (fread ((void *)&c, sizeof(struct CRYOSIN20HZ), (size_t)1, fpin)) == 1 ) {
		if(k == 0) t0 = 1.0e-06 * c.microsec + c.sec2000;
        k=k+1;
	t = 1.0e-06 * c.microsec + c.sec2000;
	lat = 1.0e-06 * c.lat;
	lon = 1.0e-06 * c.lon;
	/* tide = c.hltide + c.hstide + c.hptide + c.hotide;
        tide_csr = c.new_tide + c.hstide; */
	/*	delay = c.hwet + c.hdry + c.hiono + c.hinvb; don't use ionosphere for Cryosat > 82 deg lat and < -82 deg lat */
	delay = c.hwet + c.hdry + c.hionomod + c.hinvb;
    /* ioff = -3440.0;  based on analysis by Remko Scharroo at 2011 Crypsat meeting */
    /* issb = floor(0.060*c.pswh[1]+.5); */
	/* h0 =c.alt-(c.range+c.hdopp+tide_csr+delay+ioff)+issb; */
	swdiff = abs(c.pswh[1]-c.pswh[0]);
    
    	switch ( nstr ) {
		case 1:
			cor = c.hwet;
			break;
		case 2:
			cor = c.hdry;
			break;
		case 3:
			cor = c.hinvb;
			break;
		case 4:
			cor = c.hdopp;
			break;			
		case 5:
			cor = c.hionomod;
			break;	
		case 6:
			cor = c.hionogim;
			break;		
	}
                            
	printf("%lf %lf %d \n",lon,lat,cor);

/* dump raw waveforms */
	/*if(lat < 5. && lat > -5. && diff < 3000. && c.trackbits == 0) {
          for (kk=0; kk<128; kk++){
	    printf(" %d \n",c.wave[kk]);
          }
        }*/
	}
	fclose (fpin);
	exit (0);
}
